/*
 * Copyright 2009 Arne Vajh�j.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License. 
 */

package dk.vajhoej.record.test;

import static org.junit.Assert.*;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.junit.Test;

import dk.vajhoej.record.MapHelper;
import dk.vajhoej.record.RecordException;

public class TestMapHelper {
    @Test
    public void testFromMap() {
    	try {
			Map<String,Object> m = new HashMap<String,Object>();
			m.put("iv", 258);
			m.put("xv", 123.456);
			m.put("sv", "ABC     ");
			Data o = MapHelper.fromMap(Data.class, m);
			assertEquals("iv", 258, o.getIv());
			assertEquals("xv", 123.456, o.getXv(), 0.0005);
			assertEquals("sv", "ABC     ", o.getSv());
		} catch (RecordException e) {
			fail("Unexpected exception: " + e);
		}
    }
    @Test
    public void testToMap() {
    	try {
			Data o = new Data();
			o.setIv(258);
			o.setXv(123.456);
			o.setSv("ABC     ");
			Map<String,Object> m = MapHelper.toMap(o);
			assertEquals("iv", 258, (int)(Integer)m.get("iv"));
			assertEquals("xv", 123.456, (double)(Double)m.get("xv"), 0.0005);
			assertEquals("sv", "ABC     ", (String)m.get("sv"));
		} catch (RecordException e) {
			fail("Unexpected exception: " + e);
		}
    }
    @Test
    public void testComplexFromMap() {
    	try {
			Map<String,Object> m = new HashMap<String,Object>();
			m.put("iv", 1);
			m.put("sv", "A");
			Map<String,Object> m1 = new HashMap<String,Object>();
			m1.put("iv", 2);
			m1.put("sv", "B");
			Map<String,Object> m2 = new HashMap<String,Object>();
			m2.put("iv", 3);
			m2.put("sv", "C");
			Map<String,Object> m3 = new HashMap<String,Object>();
			m3.put("iv", 4);
			m3.put("sv", "D");
			List<Object> lst = new ArrayList<Object>();
			lst.add(m1);
			lst.add(m2);
			lst.add(m3);
			m.put("sub", lst);
			ComplexData o = MapHelper.fromMap(ComplexData.class, m);
			assertEquals("iv", 1, o.getIv());
			assertEquals("sv", "A", o.getSv());
			assertEquals("sub[0].iv", 2, o.getSub()[0].getIv());
			assertEquals("sub[0].sv", "B", o.getSub()[0].getSv());
			assertEquals("sub[1].iv", 3, o.getSub()[1].getIv());
			assertEquals("sub[1].sv", "C", o.getSub()[1].getSv());
			assertEquals("sub[2].iv", 4, o.getSub()[2].getIv());
			assertEquals("sub[2].sv", "D", o.getSub()[2].getSv());
		} catch (RecordException e) {
			fail("Unexpected exception: " + e + ": " + e.getCause());
		}
    }
    @Test
    public void testComplexToMap() {
    	try {
    		ComplexData o = new ComplexData();
    		o.setIv(1);
    		o.setSv("A");
    		ComplexSubData o1 = new ComplexSubData();
    		o1.setIv(2);
    		o1.setSv("B");
    		o.sub[0] = o1;
    		ComplexSubData o2 = new ComplexSubData();
    		o2.setIv(3);
    		o2.setSv("C");
    		o.sub[1] = o2;
    		ComplexSubData o3 = new ComplexSubData();
    		o3.setIv(4);
    		o3.setSv("D");
    		o.sub[2] = o3;
    		Map<String,Object> m = MapHelper.toMap(o);
    		assertEquals("iv", 1, m.get("iv"));
    		assertEquals("sv", "A", m.get("sv"));
		} catch (RecordException e) {
			fail("Unexpected exception: " + e + ": " + e.getCause());
		}
    }
}
