/*
 * Copyright 2016 Arne Vajhj.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License. 
 */

package dk.vajhoej.record8.test;

import static org.junit.Assert.*;

import org.junit.Test;

import dk.vajhoej.record.RecordException;
import dk.vajhoej.record.test.ArrayData;
import dk.vajhoej.record.test.VarArrayData;
import dk.vajhoej.record.test.VariableData;

import dk.vajhoej.record8.StructReader8;

public class TestStructReader {
    @Test
    public void testReadVarArray() {
        byte[] b = { 0x02, 0x01, 0x00, 0x00,
                     0x03, 0x01, 0x00, 0x00,
                     0x04, 0x01, 0x00, 0x00,
                     0x05, 0x01, 0x00, 0x00,
                     0x77, (byte)0xBE, (byte)0x9F, 0x1A, 0x2F, (byte)0xDD, 0x5E, 0x40,
                     0x77, (byte)0xBE, (byte)0x9F, 0x1A, 0x2F, (byte)0xDD, 0x5E, 0x40,
                     0x77, (byte)0xBE, (byte)0x9F, 0x1A, 0x2F, (byte)0xDD, 0x5E, 0x40,
                     0x77, (byte)0xBE, (byte)0x9F, 0x1A, 0x2F, (byte)0xDD, 0x5E, 0x40 };
        try {
            StructReader8 sr = new StructReader8(b);
            VarArrayData o = sr.read8(VarArrayData.class, (obj,n) -> 0, () -> 0, (obj,n) -> 4);
            for(int ix = 0; ix < 4; ix++) {
                assertEquals("iv", 258 + ix, o.getIv(ix));
                assertEquals("xv", 123.456, o.getXv(ix), 0.0005);
            }
            assertEquals("more", false, sr.more());
        } catch (RecordException e) {
            fail("Unexpected exception: " + e);
        }
    }
    @Test
    public void testReadVarArrayWithSizeAdjust() {
        byte[] b = { 0x02, 0x01, 0x00, 0x00,
                     0x03, 0x01, 0x00, 0x00,
                     0x04, 0x01, 0x00, 0x00,
                     0x05, 0x01, 0x00, 0x00,
                     0x77, (byte)0xBE, (byte)0x9F, 0x1A, 0x2F, (byte)0xDD, 0x5E, 0x40,
                     0x77, (byte)0xBE, (byte)0x9F, 0x1A, 0x2F, (byte)0xDD, 0x5E, 0x40,
                     0x77, (byte)0xBE, (byte)0x9F, 0x1A, 0x2F, (byte)0xDD, 0x5E, 0x40,
                     0x77, (byte)0xBE, (byte)0x9F, 0x1A, 0x2F, (byte)0xDD, 0x5E, 0x40 };
        try {
            StructReader8 sr = new StructReader8(b);
            ArrayData o = sr.read8(ArrayData.class, (obj,n) -> 0, () -> 0, (obj,n) -> 4);
            for(int ix = 0; ix < 4; ix++) {
                assertEquals("iv", 258 + ix, o.getIv(ix));
                assertEquals("xv", 123.456, o.getXv(ix), 0.0005);
            }
            assertEquals("more", false, sr.more());
        } catch (RecordException e) {
            fail("Unexpected exception: " + e);
        }
    }
    @Test
    public void testReadVariable() {
        byte[] b = { 0x41,
                     0x41, 0x42,
                     0x41, 0x42, 0x43,
                     0x41, 0x42, 0x43, 0x44 };
        try {
            StructReader8 sr = new StructReader8(b);
            VariableData o = sr.read8(VariableData.class, (obj,n) -> n + 1);
            assertEquals("s1", "A", o.getS1());
            assertEquals("s2", "AB", o.getS2());
            assertEquals("s3", "ABC", o.getS3());
            assertEquals("s4", "ABCD", o.getS4());
            assertEquals("more", false, sr.more());
        } catch (RecordException e) {
            fail("Unexpected exception: " + e);
        }
    }
}
