/*
 * Copyright 2016 Arne Vajhj.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License. 
 */

package dk.vajhoej.record8.test;

import static org.junit.Assert.*;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import org.junit.Test;

import dk.vajhoej.record.RecordException;

import dk.vajhoej.record.test.VariableData;

import dk.vajhoej.record8.Util8;

public class TestUtil8 {
    private final static int N = 10000;
    @Test
    public void testVarReadList() {
        byte[] b = { 0x41,
                     0x41, 0x42,
                     0x41, 0x42, 0x43,
                     0x41, 0x42, 0x43, 0x44 };
        byte[] bm = new byte[N*b.length];
        for(int i = 0; i < N; i++) {
            System.arraycopy(b, 0, bm, i*b.length, b.length);
        }
        try {
            List<VariableData> lst = Util8.readAll(VariableData.class, bm, (obj,n) -> n+1, () -> 20, (obj,n) -> 1);
            assertEquals("lst size", N, lst.size());
            for(int i = 0; i < lst.size(); i++) {
                assertEquals("s1", "A", lst.get(i).getS1());
                assertEquals("s2", "AB", lst.get(i).getS2());
                assertEquals("s3", "ABC",lst.get(i).getS3());
                assertEquals("s4", "ABCD", lst.get(i).getS4());
            }
        } catch (RecordException e) {
            fail("Unexpected exception: " + e);
        }
    }
    @Test
    public void testVarWriteList() {
        byte[] b = { 0x41,
                     0x41, 0x42,
                     0x41, 0x42, 0x43,
                     0x41, 0x42, 0x43, 0x44 };
        try {
            List<VariableData> lst = new ArrayList<VariableData>();
            for(int i = 0; i < N; i++) {
                VariableData o = new VariableData();
                o.setS1("A");
                o.setS2("AB");
                o.setS3("ABC");
                o.setS4("ABCD");
                lst.add(o);
            }
            byte[] res = Util8.writeAll(VariableData.class, lst, (obj,n) -> n+1, () -> 20, (obj,n) -> 1);
            assertEquals("length", N*b.length, res.length);
            for(int i = 0; i < res.length; i++) {
                assertEquals("byte " + i, b[i % b.length], res[i]);
            }
        } catch (RecordException e) {
            fail("Unexpected exception: " + e);
        }
    }
    @Test
    public void testVarReadStream() {
        byte[] b = { 0x41,
                     0x41, 0x42,
                     0x41, 0x42, 0x43,
                     0x41, 0x42, 0x43, 0x44 };
        byte[] bm = new byte[N*b.length];
        for(int i = 0; i < N; i++) {
            System.arraycopy(b, 0, bm, i*b.length, b.length);
        }
        ByteArrayInputStream bais = new ByteArrayInputStream(bm);
        try {
            List<VariableData> lst = new ArrayList<VariableData>();
            Util8.readAll(VariableData.class, bais, lst, (obj,n) -> n+1, () -> 20, (obj,n) -> 1);
            assertEquals("lst size", N, lst.size());
            for(int i = 0; i < lst.size(); i++) {
                assertEquals("s1", "A", lst.get(i).getS1());
                assertEquals("s2", "AB", lst.get(i).getS2());
                assertEquals("s3", "ABC",lst.get(i).getS3());
                assertEquals("s4", "ABCD", lst.get(i).getS4());
            }
        } catch (RecordException e) {
            fail("Unexpected exception: " + e);
        } catch (IOException e) {
            fail("Unexpected exception: " + e);
        }
    }
    @Test
    public void testVarWriteStream() {
        byte[] b = { 0x41,
                     0x41, 0x42,
                     0x41, 0x42, 0x43,
                     0x41, 0x42, 0x43, 0x44 };
        try {
            List<VariableData> lst = new ArrayList<VariableData>();
            for(int i = 0; i < N; i++) {
                VariableData o = new VariableData();
                o.setS1("A");
                o.setS2("AB");
                o.setS3("ABC");
                o.setS4("ABCD");
                lst.add(o);
            }
            ByteArrayOutputStream baos = new ByteArrayOutputStream();
            Util8.writeAll(VariableData.class, lst, baos, (obj,n) -> n+1, () -> 20, (obj,n) -> 1);
            byte[] res = baos.toByteArray();
            assertEquals("length", N*b.length, res.length);
            for(int i = 0; i < res.length; i++) {
                assertEquals("byte " + i, b[i % b.length], res[i]);
            }
        } catch (RecordException e) {
            fail("Unexpected exception: " + e);
        } catch (IOException e) {
            fail("Unexpected exception: " + e);
        }
    }
    @Test
    public void testVarReadHandler() {
        byte[] b = { 0x41,
                     0x41, 0x42,
                     0x41, 0x42, 0x43,
                     0x41, 0x42, 0x43, 0x44 };
        byte[] bm = new byte[N*b.length];
        for(int i = 0; i < N; i++) {
            System.arraycopy(b, 0, bm, i*b.length, b.length);
        }
        ByteArrayInputStream bais = new ByteArrayInputStream(bm);
        try {
            Util8.readAll(VariableData.class,
                    bais,
                    o -> {
                        assertEquals("s1", "A", o.getS1());
                        assertEquals("s2", "AB", o.getS2());
                        assertEquals("s3", "ABC", o.getS3());
                        assertEquals("s4", "ABCD", o.getS4());
                    },
                    (obj,n) -> n+1,
                    () -> 20,
                    (obj,n) -> 1);
        } catch (RecordException e) {
            fail("Unexpected exception: " + e);
        } catch (IOException e) {
            fail("Unexpected exception: " + e);
        }
    }
}
